#include <p18cxxx.h>
#include <stdio.h>

#define PORT_SET_INPUT(tris, shift) tris |= (1 << shift);
#define PORT_SET_OUTPUT(tris, shift) tris &= ~(1 << shift);

#define PORT_SET(port, shift) port |= (1 << shift);
#define PORT_CLEAR(port, shift) port &= ~(1 << shift);

//lcd
#define LCD_PORT_CS1	LATC
#define LCD_SHIFT_CS1	6
#define LCD_TRIS_CS1	TRISC

#define LCD_PORT_CS2	LATC
#define LCD_SHIFT_CS2	7
#define LCD_TRIS_CS2	TRISC

#define LCD_PORT_DI 	LATB
#define LCD_SHIFT_DI	3
#define LCD_TRIS_DI		TRISB

#define LCD_PORT_E		LATB
#define LCD_SHIFT_E		1
#define LCD_TRIS_E		TRISB

#define LCD_PORT_RW		LATB
#define LCD_SHIFT_RW	2
#define LCD_TRIS_RW		TRISB

#define LCD_PORT_DATA_OUT	LATD
#define LCD_PORT_DATA_IN	PORTD
#define LCD_MASK_DATA	0xff
#define LCD_TRIS_DATA	TRISD

#define LCD_PORT_BACKLIGHT 	LATG
#define LCD_SHIFT_BACKLIGHT	6
#define LCD_TRIS_BACKLIGHT  TRISG

#define LCD_PORT_RESET LATB
#define LCD_SHIFT_RESET 0
#define LCD_TRIS_RESET TRISB

#define E_HIGH PORT_SET(LCD_PORT_E, LCD_SHIFT_E);
#define E_LOW  PORT_CLEAR(LCD_PORT_E, LCD_SHIFT_E);

#define DI_HIGH PORT_SET(LCD_PORT_DI, LCD_SHIFT_DI);
#define DI_LOW  PORT_CLEAR(LCD_PORT_DI, LCD_SHIFT_DI);

#define RW_LOW PORT_CLEAR(LCD_PORT_RW, LCD_SHIFT_RW);
#define RW_HIGH PORT_SET(LCD_PORT_RW, LCD_SHIFT_RW);

#define CS1_LOW PORT_CLEAR(LCD_PORT_CS1, LCD_SHIFT_CS1);
#define CS1_HIGH PORT_SET(LCD_PORT_CS1, LCD_SHIFT_CS1);

#define CS2_LOW PORT_CLEAR(LCD_PORT_CS2, LCD_SHIFT_CS2);
#define CS2_HIGH PORT_SET(LCD_PORT_CS2, LCD_SHIFT_CS2);

#define LCD_DATA_OUT  LCD_TRIS_DATA &= ~(LCD_MASK_DATA);
#define LCD_DATA_IN   LCD_TRIS_DATA |= (LCD_MASK_DATA);

#define LCD_RESET_HIGH PORT_SET(LCD_PORT_RESET, LCD_SHIFT_RESET);
#define LCD_RESET_LOW PORT_CLEAR(LCD_PORT_RESET, LCD_SHIFT_RESET);

void __lcd_SelectSegment(unsigned char segment);
void __lcd_WriteInstruction(unsigned char datax);
unsigned char __lcd_ReadStatus(void);

void DelayUs(unsigned int delay_us)
{
	delay_us *= 50;
	while (delay_us--);
}

void ResetDevice(void)
{ 
//	WaitUntilBusy();
	__lcd_SelectSegment(1);
/*
	__lcd_WriteInstruction(0x40);
	__lcd_WriteInstruction(0xb8);
	__lcd_WriteInstruction(0xc0);
*/
	__lcd_WriteInstruction(0x3e);
	__lcd_WriteInstruction(0x3f);
	__lcd_SelectSegment(2);
/*
	__lcd_WriteInstruction(0x40);
	__lcd_WriteInstruction(0xb8);
	__lcd_WriteInstruction(0xc0);
*/
	__lcd_WriteInstruction(0x3e);
	__lcd_WriteInstruction(0x3f);
	__lcd_ReadStatus();
}

unsigned char lcd_init(void)
{
	PORT_SET_OUTPUT(LCD_TRIS_CS1, LCD_SHIFT_CS1);
	PORT_SET_OUTPUT(LCD_TRIS_CS2, LCD_SHIFT_CS2);
	PORT_SET_OUTPUT(LCD_TRIS_DI, LCD_SHIFT_DI);
	PORT_SET_OUTPUT(LCD_TRIS_E, LCD_SHIFT_E);
	PORT_SET_OUTPUT(LCD_TRIS_RW, LCD_SHIFT_RW);
	PORT_SET_OUTPUT(LCD_TRIS_RESET, LCD_SHIFT_RESET);

	CS1_LOW;
	CS2_LOW;
	E_LOW;
	RW_LOW;

	LCD_RESET_HIGH;
//	PORT_SET_OUTPUT(LCD_TRIS_BACKLIGHT, LCD_SHIFT_BACKLIGHT);

	LCD_DATA_OUT;
	LCD_RESET_LOW;
	DelayUs(100);
	LCD_RESET_HIGH;

//	InitGraph();
	ResetDevice();
//	PORT_SET(LCD_PORT_BACKLIGHT, LCD_SHIFT_BACKLIGHT);
	return 0;
}

void __lcd_SelectSegment(unsigned char segment)
{
	if (segment == 1)
	{
		CS1_HIGH;
		CS2_LOW;
	}
	else
	{
		CS1_LOW;
		CS2_HIGH;
	}
}

void __lcd_WriteData(unsigned char datax)
{
	DI_HIGH;
	RW_LOW;
	LCD_PORT_DATA_OUT &= ~LCD_MASK_DATA;
	LCD_PORT_DATA_OUT |= datax;
	E_HIGH;
	DelayUs(1);
	E_LOW;
}

void __lcd_WriteInstruction(unsigned char datax)
{
	DI_LOW;
	RW_LOW;
	LCD_PORT_DATA_OUT &= ~LCD_MASK_DATA;
	LCD_PORT_DATA_OUT |= datax;
	E_HIGH;
	DelayUs(1);
	E_LOW;
}

unsigned char __lcd_ReadStatus(void)
{
	unsigned int data;

	LCD_DATA_IN;
	LCD_PORT_DATA_OUT |= LCD_MASK_DATA;
	DI_LOW;
	RW_HIGH;
	E_HIGH;
	DelayUs(1);
	data = LCD_PORT_DATA_IN;
	E_LOW;
	LCD_DATA_OUT;
	
	return data;
}

void main(void)
{
	lcd_init();

}
